/* $Id$ */
/*
 * File information structure:
 *
 * byte  code
 *    0  filename body (n bytes)
 *    n  unknown (4 bytes)
 *  n+4  file size (upper 1st byte)
 *  n+5  file size (upper 2nd byte)
 *  n+6  file size (upper 3rd byte)
 *  n+7  file size (upper 4th byte)
 *  n+8  year (4th figure)
 *  n+9  year (3rd figure)
 * n+10  year (2nd figure)
 * n+11  year (1st figure)
 * n+12  month (2nd figure)
 * n+13  month (1st figure)
 * n+14  date (2nd figure)
 * n+15  date (1st figure)
 * n+16  hour (2nd figure)
 * n+17  hour (1st figure)
 * n+18  minute (2nd figure)
 * n+19  minute (1st figure)
 * n+20  second (2nd figure)
 * n+21  second (1st figure)
 * n+22  tick? (2nd figure)
 * n+23  tick? (1st figure)
 *
 * charset: Shift_JIS
 */
#include <time.h>
#include "common.h"
#include "utils.h"
#include "fileinfo.h"

#define FILENAME_LEN 256

struct fileInfo {
  char *filename; /* filename */
  int unknown;    /* ? */
  int size;       /* size */
  time_t time;    /* time stamp */
  int usec;
};

static int digit_to_int(const unsigned char *p, size_t n);


FileInfo *fileInfo_new(const unsigned char *p)
{
  FileInfo *info;
  char *name;
  struct tm t;
  int year;

  info = XMALLOC(FileInfo, 1);
  info->filename = XMALLOC(char, FILENAME_LEN);
  name = info->filename;
  while (*p != '\0') {
    *(name++) = *(p++);
  }
  *name = *(p++);
  info->unknown = uchar4_to_int(p);
  info->size = uchar4_to_int(p+4);
  year = digit_to_int(p+8, 4);
  if (year >= 1970) {
    t.tm_year = year-1900;
    t.tm_mon  = digit_to_int(p+12, 2)-1;
    t.tm_mday = digit_to_int(p+14, 2);
    t.tm_hour = digit_to_int(p+16, 2);
    t.tm_min  = digit_to_int(p+18, 2);
    t.tm_sec  = digit_to_int(p+20, 2);
    t.tm_isdst = 0;
    info->time = mktime(&t);
    info->usec = digit_to_int(p+22, 2);
  } else {
    /* return epoch */
    info->time = 0;
    info->usec = 0;
  }
  return info;
}

void fileInfo_delete(FileInfo *p)
{
  XFREE(p->filename);
  XFREE(p);
}

const char *fileInfo_filename(FileInfo *info)
{
  return info->filename;
}

int fileInfo_unknown(FileInfo *info)
{
  return info->unknown;
}

int fileInfo_size(FileInfo *info)
{
  return info->size;
}

time_t fileInfo_time(FileInfo *info)
{
  return info->time;
}

int fileInfo_usec(FileInfo *info)
{
  return info->usec;
}

static int digit_to_int(const unsigned char *p, size_t n)
{
  int i;

  i = 0;
  while (n-- > 0) {
    i *= 10;
    i += *(p++)-'0';
  }
  return i;
}
