/* $Id$ */
/*
 * Filename array structure:
 *
 * byte  code
 *    0  number of files (upper byte)
 *    1  number of files (lower byte)
 *    2  filename[0] body (n[0] bytes)
 *  n+2  NUL
 *    .  filename[1] body (n[1] bytes)
 *    .  NUL
 *    .  ...
 *    .  filename[m] body (n[m] bytes)
 *    .  NUL
 *
 * charset: Shift_JIS
 */
#include "common.h"
#include "utils.h"
#include "filelist.h"

struct fileList {
  unsigned char *data;   /* data body */
  unsigned char **names; /* pointer to filename */
  size_t length;         /* length of list */
};


FileList *fileList_new(const unsigned char *src, size_t len)
{
  FileList *flist;
  unsigned char *dst, *pdst, **list, **plist;
  size_t i, j, num;

  flist = XMALLOC(FileList, 1);
  num = uchar2_to_int(src);
  src += 2;
  len -= 2;
  pdst = dst = XMALLOC(unsigned char, len);
  plist = list = XMALLOC(unsigned char *, num);
  i = j = 0;
  while (i < len) {
    *(plist++) = pdst;
    while (*src != '\0') {
      *(pdst++) = *(src++);
      i++;
    }
    *(pdst++) = *(src++);
    i++;
    j++;
  }
  flist->names = list;
  flist->length = num;
  flist->data = dst;
  return flist;
}

void fileList_delete(FileList *p)
{
  XFREE(p->names);
  XFREE(p->data);
  XFREE(p);
}

const char *fileList_get(FileList *list, size_t index)
{
  size_t num;
  char **name;

  num = list->length;
  name = list->names;
  if (index > num) {
    return NULL;
  }
  return *(name+index);
}

size_t fileList_size(FileList *list)
{
  return list->length;
}
