/* $Id$ */
#include <string.h> /* memcpy(), strlen() */
#include "common.h"
#include "utils.h"
#include "framedata.h"

#define TYPE_COMMAND 0x00
#define TYPE_DATA    0x80


FrameData *frameData_set_protocol_version_new(void)
{
  unsigned char cmd[] = {
    0x00, 0x00,              /* SetProtocolVersion */
    0x00, 0x01, 0x00, 0x00   /* version = 0x00010000 */
  };
  return frameData_new(TYPE_COMMAND, cmd, 6);
}

FrameData *frameData_set_baudrate_new(int rate)
{
  unsigned char cmd[] = {
    0x00, 0x01,              /* SetBaudRate */
    0x00, 0x00, 0x02, 0x58   /* baudrate = 0x00000258 = 600 */
  };
  int_to_uchar4(cmd+2, rate);
  return frameData_new(TYPE_COMMAND, cmd, 6);
}

FrameData *frameData_set_framesize_new(int size)
{
  unsigned char cmd[] = {
    0x00, 0x03,              /* SetFrameSize */
    0x00, 0x00, 0x02, 0x00   /* flamesize = 0x00000200 = 512 */
  };
  int_to_uchar4(cmd+2, size);
  return frameData_new(TYPE_COMMAND, cmd, 6);
}

FrameData *frameData_set_file_new(const char *name)
{
  unsigned char cmd0[] = {
    0x00, 0x10 /* SetFile */
  };
  unsigned char *cmd;
  size_t len;

  len = strlen(name);
  cmd = XMALLOC(unsigned char, len+3);
  memcpy(cmd, cmd0, 2);
  memcpy(cmd+2, name, len+1);
  return frameData_new(TYPE_COMMAND, cmd, len+3);
}

FrameData *frameData_put_file_new(const char *name, size_t size)
{
  unsigned char cmd0[] = {
    0x01, 0x10 /* PutFile */
  };
  unsigned char *cmd;
  size_t len;

  len = strlen(name);
  cmd = XMALLOC(unsigned char, len+7);
  memcpy(cmd, cmd0, 2);
  memcpy(cmd+2, name, len+1);
  int_to_uchar4(cmd+len+3, size);
  return frameData_new(TYPE_COMMAND, cmd, len+7);
}

FrameData *frameData_get_file_new(const char *name)
{
  unsigned char cmd0[] = {
    0x02, 0x10 /* GetFile */
  };
  unsigned char *cmd;
  size_t len;

  len = strlen(name);
  cmd = XMALLOC(unsigned char, len+3);
  memcpy(cmd, cmd0, 2);
  memcpy(cmd+2, name, len+1);
  return frameData_new(TYPE_COMMAND, cmd, len+3);
}

FrameData *frameData_get_file_info_new(void)
{
  unsigned char cmd[] = {
    0x02, 0x12 /* GetFileInfo */
  };
  return frameData_new(TYPE_COMMAND, cmd, 2);
}

FrameData *frameData_get_file_list_new(void)
{
  unsigned char cmd[] = {
    0x02, 0x14 /* GetFileList */
  };
  return frameData_new(TYPE_COMMAND, cmd, 2);
}


FrameData *frameData_data_new(const unsigned char *buf, size_t len)
{
  return frameData_new(TYPE_DATA, buf, len);
}

FrameData *frameData_new(unsigned char type, const unsigned char *buf, size_t len)
{
  FrameData *p;
  unsigned char *data;

  data = XMALLOC(unsigned char, len);
  memcpy(data, buf, len);
  p = XMALLOC(FrameData, 1);
  p->type = type;
  p->data = data;
  p->length = len;
  return p;
}

void frameData_delete(FrameData *p)
{
  XFREE(p->data);
  XFREE(p);
}
