/* $Id$ */
#include <string.h> /* memset() */
#include <termios.h>
#include "serial_term.h"

struct terminal_speed {
  speed_t speed;
  int rate;
};

static speed_t term_baudrate_to_speed(int rate);


void term_initialize(int fd, struct termios *oldtio)
{
  struct termios newtio;

  tcgetattr(fd, oldtio);
  memset(&newtio, 0, sizeof(struct termios));
  newtio.c_cflag = B600 | CS8 | PARENB | CLOCAL | CREAD;
  newtio.c_iflag = IGNPAR; /* ignore bytes with parity errors */
  newtio.c_oflag = 0;      /* raw output */
  newtio.c_lflag = 0;      /* set input mode */
  newtio.c_cc[VTIME] = 0;  /* inter-character timer unused */   
  newtio.c_cc[VMIN] = 1;   /* blocking read until 1 character arrives */
  tcflush(fd, TCIFLUSH);
  tcsetattr(fd, TCSANOW, &newtio);
}

void term_restore(int fd, struct termios *oldtio)
{
  tcsetattr(fd, TCSANOW, oldtio); /* restore port settings */
}

void term_set_baudrate(int fd, int rate)
{
  struct termios tio;
  speed_t speed;

  tcgetattr(fd, &tio);
  speed = term_baudrate_to_speed(rate);
  cfsetospeed(&tio, speed);
  cfsetispeed(&tio, speed);
  tcflush(fd, TCIFLUSH);
  tcsetattr(fd, TCSANOW, &tio);
}

static speed_t term_baudrate_to_speed(int rate)
{
  struct terminal_speed speeds[] = {
    { B230400, 230400 },
    { B115200, 115200 },
    { B57600, 57600 },
    { B38400, 38400 },
    { B19200, 19200 },
    { B9600, 9600 },
    { B4800, 4800 },
    { B2400, 2400 },
    { B1200, 1200 },
    { B600, 600 },
    { B300, 300 }
  };
  struct terminal_speed *p;

  if (rate < 600) {
    return B600;
  }
  /* liner search */
  p = speeds;
  while (p->rate > rate) {
    p++;
  }
  return p->speed;
}
