/* $Id$ */
#include <errno.h>
#include "common.h"
#include "strconv.h"

#if defined ENABLE_ICONV
#include <iconv.h>
#define xiconv_open(tocode,fromcode) iconv_open(tocode,fromcode)
#define xiconv_close(cd) iconv_close(cd)
#define xiconv(cd,inbuf,inbytesleft,outbuf,outbytesleft) iconv(cd,inbuf,inbytesleft,outbuf,outbytesleft)
#endif /* ENABLE_ICONV */

enum strconvCode {
  JA_RAW_CODE = -1,
  JA_EUC_JP = 0,
  JA_SHIFT_JIS,
  JA_ISO_2022_JP,
  JA_UTF_8,
  JA_CODE_END
};
static char *iconv_codes[] = {
  "EUC-JP",
  "SHIFT_JIS",
  "ISO_2022_JP",
  "UTF_8"
};
static int ja_code = JA_RAW_CODE;

static int iconv_wrapper(const char *tocode, const char *fromcode, char *dst, size_t dst_len, const char *src);


void strconv_set_code(int code)
{
  ja_code = code;
}

int strconv_to_term(char *dst, size_t dst_len, const char *src)
{
  char *tocode, *fromcode;
  tocode = iconv_codes[JA_SHIFT_JIS];
  if ((ja_code < 0) || (ja_code >= JA_CODE_END)) {
    return FALSE;
  }
  fromcode = iconv_codes[ja_code];
  return iconv_wrapper(tocode, fromcode, dst, dst_len, src);
}

int strconv_from_term(char *dst, size_t dst_len, const char *src)
{
  char *tocode, *fromcode;
  fromcode = iconv_codes[JA_SHIFT_JIS];
  if ((ja_code < 0) || (ja_code >= JA_CODE_END)) {
    return FALSE;
  }
  tocode = iconv_codes[ja_code];
  return iconv_wrapper(tocode, fromcode, dst, dst_len, src);
}


static int iconv_wrapper(const char *tocode, const char *fromcode, char *dst, size_t dst_len, const char *src)
{
  iconv_t cd;
  char *inbuf, *outbuf;
  size_t inbytesleft, outbytesleft, size;
  int rc = FALSE;

  inbuf = (char *)src;
  outbuf = dst;
  inbytesleft = strlen(inbuf)+1;
  outbytesleft = dst_len;
  cd = xiconv_open(tocode, fromcode);
  if (cd == (iconv_t)(-1)) {
    DEBUG_PRINT("iconv_open() failed. to=%s, from=%s.\n", tocode, fromcode);
    return FALSE;
  }
  size = xiconv(cd, &inbuf, &inbytesleft, &outbuf, &outbytesleft);
  if (size < 0) {
    perror("iconv()");
  }
  switch (errno) {
    case E2BIG:
      perror("E2BIG");
      rc = FALSE;
      break;
    case EILSEQ:
      perror("EILSEQ");
      rc = FALSE;
      break;
    case EINVAL:
      perror("EINVAL");
      rc = FALSE;
      break;
    default:
      rc = TRUE;
      break;
  }
  if (xiconv_close(cd)) {
    DEBUG_PRINT("iconv_close() failed.\n");
  }
  return rc;
}
