/* $Id$ */
#include "utils.h"


/*
 * integer to 4bits*2 big endian.
 */
void int_to_uchar4bits2(unsigned char *p, int v)
{
  *(p++) = (v >> 4) & 0x0f;
  *p = v & 0x0f;
}

/*
 * 16bits to 8bits*2 big endian.
 */
void int_to_uchar2(unsigned char *p, int v)
{
  *(p++) = (v >> 8) & 0xff;
  *p = v & 0xff;
}

/*
 * 32bits to 8bits*4 big endian.
 */
void int_to_uchar4(unsigned char *p, int v)
{
  *(p++) = (v >> 24) & 0xff;
  *(p++) = (v >> 16) & 0xff;
  *(p++) = (v >> 8) & 0xff;
  *p = v & 0xff;
}

/*
 * 8bits*2 big endian to 32bits int.
 */
int uchar2_to_int(const unsigned char *p)
{
  int hi, lo;

  hi = *(p++);
  lo = *p;
  return (hi << 8) | lo;
}

/*
 * 8bits*4 big endian to 32bits int.
 */
int uchar4_to_int(const unsigned char *p)
{
  int v, i;

  v = 0x0;
  for (i = 0; i < 3; i++) {
    v += *(p++);
    v <<= 8;
  }
  v += *p;
  return v;
}

/*
 * 4bits*2 big endian to 8bits.
 */
unsigned char uchar4bits2_to_uchar(const unsigned char *p)
{
  unsigned char hi, lo;

  hi = *(p++);
  lo = *p & 0x0f;
  return ((hi << 4) & 0xf0) | lo;
}
